// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller verwenden
// AVR-Version / GGC 4.3.2
// LCD DIP204-4 4x20 (bei Bedarf)
// Version 1.0 / Stand: 01.11.2009
// AVR-Version / GGC 4.3.2
// Mikrocontroller: ATmega644 (andere mglich)
// Frequenz: mit 1MHz und 8MHz getestet
// VNC1L-Interface: SPI (per Software)
// ********************************************************************
// FLASH-Speicherbedarf: ca. 5.3 kByte (inkl. LCD-Funktionen)
//                       ca. 4.2 kByte (ohne LCD-Funktionen)
// SRAM-Speicherbedarf:  ca. 700 Byte (Einsparpotential vorhanden!)
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// Hinweis: Exotische Sticks (z.B. DualCanal OZC) lassen sich vom VNC1L
// offenbar nicht beschreiben.
// Getestete USB-Sticks:
// RUNDISK 512MB FAT16 - ok
// PLATINUM  2GB FAT32 - ok
// CNMEMORY  4GB FAT32 - ok
// TREKSTOR 16GB FAT32 - ok
// OCZ 8GB (RALLY DUAL-CANAL) FAT32 -  kein schreiben mglich
// (VDAP-Firmware kann mit Dual-Canal nicht korrekt umgehen?)
// ********************************************************************
// Aktuelle VDAP Firmware muss auf den VDIP1/VNC1L installiert sein,
// getestet mit Version 3.68. Achtung: Auf neuen VDIP1 ist meist eine
// sehr alte Version drauf, also erstmal ein Firmware-Update machen.
// ********************************************************************
// Das Programm schreibt 2 Dateien auf den USB-Stick. Eine Textdatei
// und eine Binrdatei und zeigt die Verwendung weiterer Kommandos
// bzw. auch das einlesen einer Datei. Ohne LCD sollte die LED nach
// ca. 10s blinken. Wenn ein Fehler auftritt, leuchtet die LED dauerhaft
// und das Programm wird angehalten. Mit LCD werden alle Schritte
// per Monitoring angezeigt. Fr die Inbetriebnahme sehr hilfreich.
// ********************************************************************
//
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR-Typ: ATmega644
//************************************************************************
//
//************************************************************************
// Includebereich
//************************************************************************
#include "mydefs.h"
#include <avr/io.h>         // IO-Grundfunktionen
#include <string.h>		    // Stringfunktion
#include <stdio.h>		    // Standardfunktion
#include <stdlib.h>         // Standardbibliothek
#include <util/delay.h>     // Pasuenfunktionen

//************************************************************************
// Definebereich ANPASSEN
//************************************************************************

// Portverwendung ATmega644
// LED: PB0
// VNC: SCLK PD3
//      SDI  PD4
//      SDO  PD5
//      CS   PD6
//      RS   PD7
//
// optional
// LCD: Data Port A (8 Bit-Interface)
//      E    PB1
//      RW   PB2
//      RS   PB3

#define LED_D1_EIN  PORTB |=  (1<<PB0)     /* LED */
#define LED_D1_AUS  PORTB &= ~(1<<PB0)

#define SCLK_L  PORTD &= ~(1<<PD3)  /* SCLK VNC1L */
#define SCLK_H  PORTD |=  (1<<PD3)

#define SDI_L   PORTD &= ~(1<<PD4)  /* SDI  VNC1L */
#define SDI_H   PORTD |=  (1<<PD4)

#define SDO_VNC (PIND & (1<<PD5)) 	/* SDO  VNC1L */

#define CS_VNC_L PORTD &= ~(1<<PD6) /* CS   VNC1L */
#define CS_VNC_H PORTD |=  (1<<PD6)

#define RE_VNC_L PORTD &= ~(1<<PD7) /* RS   VNC1L */
#define RE_VNC_H PORTD |=  (1<<PD7)

#define PROMPT_CHECK           2
#define PROMPT_UND_ERROR_CHECK 1
#define PROMPT_OK              1
#define FILE_NEW               0
#define FILE_APPEND            1

//************************************************************************
// Variablenbereich
//************************************************************************
struct vnc_struc {
  char rec_buffer[40];       // Empfangsbuffer letzte Daten
  char send_buffer[40];      // Sendebuffer
  unsigned char status;      // Status der Empfangs-Daten
  unsigned char error;       // VNC Fehler
};
struct vnc_struc vnc;  // VNC Datensatz 

// Textfile zum testen (Zeigerfeld)
char txt_z1[]  = "Systemkonfiguration\x00D\x00A";
char txt_z2[]  = "V1.0 vom 01.11.2009\x00D\x00A";
char txt_z3[]  = "Device: ATmega644\x00D\x00A";
char txt_z4[]  = "Frequenz: 1MHz/8MHz\x00D\x00A";
char txt_z5[]  = "Hardware: EVA-Board\x00D\x00A";
char txt_z6[]  = "USB Hst: VDIP1 VNC1L\x00D\x00A";
char txt_z7[]  = "Firmware: 03.68 VDAP\x00D\x00A";
char txt_z8[]  = "Anzeige: DIP204 4x20\x00D\x00A";
char txt_z9[]  = "Compiler: GCC 4.3.2\x00D\x00A";
char txt_z10[] = "Prg.: ISP\x00D\x00A";
char txt_z11[] = "\x0FF"; // File-Endekennung
char *file1[] = {
	txt_z1,	txt_z2,	txt_z3,	txt_z4,	 txt_z5, txt_z6,
	txt_z7,	txt_z8,	txt_z9,	txt_z10, txt_z11 };

// Binrfile zum testen
char file2[] = {
  0xFA,0x89,0x11,0x23,0xAA,0x02,0x00,0x1F,
  0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
  0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
  0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
  0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
  0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
  0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
  0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
  0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
  0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC };

char test_buffer[100]; // Testbuffer fr eingelesenes File, Lnge wird nicht geprft!!!

//************************************************************************
// USART-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
#define USART_MONITORING /* aus- / einkommentieren */

#ifdef  USART_MONITORING
  #include "usart.h"
#endif

//************************************************************************
// LCD-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
//#define LCD_MONITORING /* aus- / einkommentieren */
#ifdef  LCD_MONITORING /* LCD-Funktionen bei Bedarf */
  #define LCD_E1_L PORTB &= ~(1<<PB1)
  #define LCD_E1_H PORTB |=  (1<<PB1)

  #define LCD_RW_L PORTB &= ~(1<<PB2)
  #define LCD_RW_H PORTB |=  (1<<PB2)

  #define LCD_RS_L PORTB &= ~(1<<PB3)
  #define LCD_RS_H PORTB |=  (1<<PB3)

  #define LCD_DATA      PORTA
  #define LCD_DATA_READ PINA
  #define LCD_PORT      DDRA

  #define SPALTEN_MAX  20
  #define ZEILEN_MAX    4
  #define SCOLL_DELAY 200
  struct lcd_struc {
    unsigned char x;  // Spalte 
    unsigned char y;  // Zeile
  };
  struct lcd_struc lcd;   // LCD Datensatz 
  char lcd_buffer[25];
  #include "dip204_avr.c" // LCD-Basis-Funktionen fr DIP204-4 KS0073
#endif

#include "vnc1l_avr.c" // VNC1L-Basis-Funktionen

// ********************************************************************
// System-Initialisierungen ANPASSEN
// ********************************************************************
void init_system (void) {
  // Portdefinition
  // LED
  DDRB  |= (1 << PB0); // PB0 als Ausgang
  LED_D1_AUS;

  // VNC1L
  DDRD  |= ((1 << PD3) | (1 << PD4) | (1 << PD6) | (1 << PD7)); // PD3,PD4,PD6,PD7 als Ausgang
  DDRD  &= ~(1 << PD5); // PD5 als Eingang

#ifdef  LCD_MONITORING  
  // LCD
  DDRA   = 0xFF; // Port A alle PA0 ... PA7  als Ausgang
  DDRB  |= ((1 << PB1) | (1 << PB2) | (1 << PB3)); // PB1,PB2,PB3 als Ausgang
#endif

#ifdef  USART_MONITORING
  usart_init(BAUDRATE); // setup the UART
#endif
}

// ********************************************************************
// Hauptprogramm
// ********************************************************************
int main (void) {
  init_system();
#ifdef LCD_MONITORING
  init_lcd();     // LCD-Initialisierung 
  printf_lcd(0,0,"VNC1L-1A -Test-",1); // Einschaltmeldung  
  printf_lcd(0,1,"AVR Atmega644",1);  
  printf_lcd(0,2,"DIP204 (KS0073)",1);
  printf_lcd(0,3,"V1.0 01.11.2009 GCC",1);
  _delay_ms(2000);  // Pause
  clear_lcd();      // LCD lschen
#endif
  vnc_init();          // VNC1L-Initialisierung
  vnc.error = 0;       // Fehlerflag rcksetzen 0=ok 1,2,3,4....=Fehler-Nr.
  
  vnc_prompt_check(); // auf erstes Prompt warten
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch

  vnc_wr_cmd("ECS");  // Extented Command Set
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch

  vnc_wr_cmd("IPA"); // Monitor Commands Value use ASCII
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch

  vnc_wr_cmd("FWV"); // FWV Firmwareversion anzeigen
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch

  vnc_wr_cmd("DIR"); // Dir Command
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch
/*
  vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_NEW);    // Textfile auf USB-Stick erzeugen
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch
    
//  vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_APPEND); // gleiches Textfile auf USB-Stick nochmal anhngen
//  vnc_prompt_check(); // Besttigung prfen
//  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch  
  
  vnc_wr_binfile("VNCFILE2.BIN",file2,80);             // Binrfile auf USB-Stick erzeugen
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch
    
  vnc_rd_file("VNCFILE1.TXT",test_buffer,40);          // File (Teil) von USB-Stick in Speicher einlesen
  vnc_prompt_check(); // Besttigung prfen
  if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch
*/  
  // kein Fehler --> die LED blinkt
  // USB-Stick kann entnommen werden
  sei(); 
  while(1) {/*
    LED_D1_EIN;
    _delay_ms(1000);
    LED_D1_AUS;
    _delay_ms(1000);*/

#ifdef  USART_MONITORING
	if (usart_status.usart_ready) {
      vnc_wr_cmd(usart_rx_buffer);
      vnc_prompt_check(); // Besttigung prfen
      if (vnc.error) vnc_fehlermeldung(); // wenn negativ Fehlermeldung/Abbruch
	}
	usart_status.usart_ready =0;
    vnc_rd_byte(0);
#endif

  }
}

